"use strict";
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __metadata = (this && this.__metadata) || function (k, v) {
    if (typeof Reflect === "object" && typeof Reflect.metadata === "function") return Reflect.metadata(k, v);
};
var __param = (this && this.__param) || function (paramIndex, decorator) {
    return function (target, key) { decorator(target, key, paramIndex); }
};
var WavefrontProxyCheckinController_1;
Object.defineProperty(exports, "__esModule", { value: true });
exports.WavefrontProxyCheckinController = void 0;
const common_1 = require("@nestjs/common");
const wavefront_proxy_attributes_service_1 = require("../../services/wavefront-proxy-attributes/wavefront-proxy-attributes.service");
const metric_record_processor_service_1 = require("../../services/metric-record-processor/metric-record-processor.service");
let WavefrontProxyCheckinController = WavefrontProxyCheckinController_1 = class WavefrontProxyCheckinController {
    constructor(wfpAttributes, metricRecordProcessor) {
        this.wfpAttributes = wfpAttributes;
        this.metricRecordProcessor = metricRecordProcessor;
        this.logger = new common_1.Logger(WavefrontProxyCheckinController_1.name);
    }
    checkIn(query, body, headers) {
        try {
            this.wfpAttributes.updateProxyAttributes(query, headers);
            const topLevelMetrics = Object.keys(body);
            const proxyTags = {
                source: query['hostname'] ?? 'Unknown Host',
                wfp_hostname: query['hostname'] ?? 'Unknown Host',
                wfp_version: query['version'] ?? 'Unknown Version',
                wfp_name: query['proxyname'] ?? 'Unknown Name',
            };
            this.logger.debug(`Received checkin from proxy ${proxyTags.wfp_name}@${proxyTags.wfp_hostname}`);
            const proxyName = query['proxyname'];
            topLevelMetrics.forEach((metric) => {
                const metricNames = new Array();
                this.findMetrics([metric], body[metric].value, metricNames);
                metricNames.forEach((metricParts) => {
                    const value = this.getValue(body, metricParts);
                    const metricTags = this.getTags(body, metricParts);
                    const attributeName = this.getAttributeName(metricParts);
                    const metricRecord = {
                        metric: attributeName.indexOf('$') > 0
                            ? attributeName.substring(0, attributeName.indexOf('$'))
                            : attributeName,
                        rawValue: '' + value,
                        timestamp: '' + new Date().getTime(),
                        tags: { ...proxyTags, ...metricTags },
                    };
                    this.metricRecordProcessor.processRecord(metricRecord);
                });
            });
            return { name: proxyName, currentTime: new Date().getTime() };
        }
        catch (error) {
            this.logger.error('Error processing checkin', error);
            return { error: true };
        }
    }
    getTags(body, metricName) {
        return body[metricName[0]].tags;
    }
    getValue(body, metricName) {
        if (metricName.length == 1) {
            return body[metricName[0]].value;
        }
        if (metricName.length == 2) {
            return body[metricName[0]].value[metricName[1]];
        }
        if (metricName.length == 3) {
            return body[metricName[0]].value[metricName[1]][metricName[2]];
        }
        if (metricName.length == 4) {
            return body[metricName[0]].value[metricName[1]][metricName[2]][metricName[3]];
        }
        return 'too many nestings';
    }
    findMetrics(parent, value, metricNames) {
        if (value == null) {
            return metricNames;
        }
        if (typeof value === 'number' || typeof value === 'string') {
            metricNames.push(parent);
            return metricNames;
        }
        if (typeof value === 'object') {
            const thisLevelMetrics = Object.keys(value);
            thisLevelMetrics.forEach((metric) => {
                const nm = Object.assign([], parent);
                nm.push(metric);
                this.findMetrics(nm, value[metric], metricNames);
            });
            return metricNames;
        }
        else
            this.logger.warn('metric value was not recognized');
    }
    getAttributeName(metricParts) {
        return metricParts.join('.');
    }
};
exports.WavefrontProxyCheckinController = WavefrontProxyCheckinController;
__decorate([
    (0, common_1.Post)('/api/v*/wfproxy/checkin'),
    __param(0, (0, common_1.Query)()),
    __param(1, (0, common_1.Body)()),
    __param(2, (0, common_1.Headers)()),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [Object, Object, Object]),
    __metadata("design:returntype", Object)
], WavefrontProxyCheckinController.prototype, "checkIn", null);
exports.WavefrontProxyCheckinController = WavefrontProxyCheckinController = WavefrontProxyCheckinController_1 = __decorate([
    (0, common_1.Controller)(),
    __metadata("design:paramtypes", [wavefront_proxy_attributes_service_1.WavefrontProxyAttributesService,
        metric_record_processor_service_1.MetricRecordProcessorService])
], WavefrontProxyCheckinController);
//# sourceMappingURL=wavefront-proxy-checkin.controller.js.map