"use strict";
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __metadata = (this && this.__metadata) || function (k, v) {
    if (typeof Reflect === "object" && typeof Reflect.metadata === "function") return Reflect.metadata(k, v);
};
var __param = (this && this.__param) || function (paramIndex, decorator) {
    return function (target, key) { decorator(target, key, paramIndex); }
};
var WavefrontProxyReportController_1;
Object.defineProperty(exports, "__esModule", { value: true });
exports.WavefrontProxyReportController = void 0;
const common_1 = require("@nestjs/common");
const readline = require("node:readline");
const node_stream_1 = require("node:stream");
const otsdb_line_processor_service_1 = require("../../services/otsdbline-processor/otsdb-line-processor.service");
const wavefront_proxy_attributes_service_1 = require("../../services/wavefront-proxy-attributes/wavefront-proxy-attributes.service");
const config_1 = require("@nestjs/config");
const fs = require("node:fs");
let WavefrontProxyReportController = WavefrontProxyReportController_1 = class WavefrontProxyReportController {
    constructor(lineProcessor, wfpAttributes, configuration) {
        this.lineProcessor = lineProcessor;
        this.wfpAttributes = wfpAttributes;
        this.configuration = configuration;
        this.logger = new common_1.Logger(WavefrontProxyReportController_1.name);
        this.debugReportingIngestion = false;
        if (this.configuration.get('adapter.debug.debugReportingIngestion')) {
            const debugReportingIngestionFile = this.configuration.get('adapter.debug.debugReportingIngestionFile');
            try {
                this.logger.debug(`Saving ingestion log to ${debugReportingIngestionFile}`);
                this.intializeDebugReportingIngestionStream(debugReportingIngestionFile);
            }
            catch (error) {
                this.logger.error(`No access to specified ingestion reporting file: ${debugReportingIngestionFile}`, error);
            }
        }
        else {
            this.logger.debug('Reporting Ingestion Debug file is not enabled. Enable it by setting env variable DX_WFP_ADAPTER_DEBUG_REPORTING_INGESTION_FILE to a writable file.');
        }
    }
    intializeDebugReportingIngestionStream(debugReportingIngestionFile) {
        this.debugReportingIngestionStream = fs.createWriteStream(debugReportingIngestionFile, 'utf-8');
        this.debugReportingIngestionStream.on('error', (error) => {
            this.logger.error('Error writing to debug reporting ingestion stream', error);
        });
        this.debugReportingIngestion = true;
    }
    async reportMetrics(headers, query, req) {
        {
            return new Promise((resolve, reject) => {
                const stream = node_stream_1.Readable.from(req.body, {
                    objectMode: false,
                    highWaterMark: 16,
                });
                stream.on('error', (error) => {
                    this.logger.error('Error processing stream', error);
                    reject(error);
                });
                const lineReader = readline.createInterface(stream);
                let linesRead = 0;
                lineReader.on('line', (line) => {
                    try {
                        this.debugIngestion(line);
                        this.lineProcessor.processLine(line, headers['x-wf-proxy-id']);
                        linesRead++;
                    }
                    catch (error) {
                        this.logger.error(`Error processing line: '${line}'`, error);
                    }
                });
                lineReader.on('close', () => {
                    this.logger.debug(`Report request completed with ${linesRead} lines read.`);
                    resolve({
                        linesRead,
                    });
                });
                lineReader.on('error', (error) => {
                    this.logger.error('Error processing lines', error);
                    reject(error);
                });
            });
        }
    }
    debugIngestion(line) {
        if (this.debugReportingIngestion &&
            this.debugReportingIngestionStream &&
            !this.debugReportingIngestionStream.closed &&
            !this.debugReportingIngestionStream.destroyed) {
            this.debugReportingIngestionStream.write(line + '\n');
        }
    }
};
exports.WavefrontProxyReportController = WavefrontProxyReportController;
__decorate([
    (0, common_1.Post)('/api/v*/wfproxy/report'),
    __param(0, (0, common_1.Headers)()),
    __param(1, (0, common_1.Query)()),
    __param(2, (0, common_1.Req)()),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [Object, String, Request]),
    __metadata("design:returntype", Promise)
], WavefrontProxyReportController.prototype, "reportMetrics", null);
exports.WavefrontProxyReportController = WavefrontProxyReportController = WavefrontProxyReportController_1 = __decorate([
    (0, common_1.Controller)(),
    __metadata("design:paramtypes", [otsdb_line_processor_service_1.OTSDBLineProcessorService,
        wavefront_proxy_attributes_service_1.WavefrontProxyAttributesService,
        config_1.ConfigService])
], WavefrontProxyReportController);
//# sourceMappingURL=wavefront-proxy-report.controller.js.map