"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.cliOptions = void 0;
const core_1 = require("@nestjs/core");
const app_module_1 = require("./app.module");
const compression = require("compression");
const common_1 = require("@nestjs/common");
const cli_1 = require("./util/cli/cli");
const path = require("node:path");
const adapter_version_service_1 = require("./services/adapter-version/adapter-version.service");
async function bootstrap() {
    common_1.Logger.log('Current installation Directory: ' + path.resolve(__dirname, '..'));
    const logLevels = (exports.cliOptions.logLevels ||
        process.env.DX_WFPA_LOG_LEVELS ||
        'error,warn,log')
        .split(',')
        .map((level) => level.trim());
    common_1.Logger.log(`DX Wavefront Proxy Adaptor Log Levels: ${logLevels}`, 'Bootstrap');
    const app = await core_1.NestFactory.create(app_module_1.AppModule, {
        logger: logLevels,
    });
    app.use(compression());
    const dx_wfpa_version = app
        .get(adapter_version_service_1.AdapterVersionService)
        .getVersion();
    common_1.Logger.log('Instantiated Components', 'Bootstrap');
    const requestBodyMax = exports.cliOptions.requestBodyMaxSize ||
        process.env.DX_WFPA_REQUEST_BODY_BYTES_MAX ||
        '17mb';
    app.useBodyParser('text', {
        limit: requestBodyMax,
    });
    app.useBodyParser('json', {
        limit: requestBodyMax,
    });
    common_1.Logger.log(`Incoming Request Body Limit set to ${requestBodyMax}`, 'Bootstrap');
    const listenPort = exports.cliOptions.listenPort || process.env.DX_WFPA_LISTEN_PORT || 2401;
    await app.listen(listenPort);
    console.log('                                                                               \n' +
        '         8I                                     ,dPYb,                         \n' +
        "         8I                                     IP'`Yb                         \n" +
        '         8I                                     I8  8I                         \n' +
        "         8I                                     I8  8'                         \n" +
        '   ,gggg,8I     ,gg,   ,gg     gg    gg    gg   I8 dP   gg,gggg,      ,gggg,gg \n' +
        '  dP"  "Y8I    d8""8b,dP"      I8    I8    88bg I8dP    I8P"  "Yb    dP"  "Y8I \n' +
        " i8'    ,8I   dP   ,88\"        I8    I8    8I   I8P     I8'    ,8i  i8'    ,8I \n" +
        ',d8,   ,d8b,,dP  ,dP"Y8,      ,d8,  ,d8,  ,8I  ,d8b,_  ,I8 _  ,d8\' ,d8,   ,d8b,\n' +
        'P"Y8888P"`Y88"  dP"   "Y88    P""Y88P""Y88P"   PI8"8888PI8 YY88888PP"Y8888P"`Y8\n' +
        '                                                I8 `8,  I8                     \n' +
        '                                                I8  `8, I8                     \n' +
        '                                                I8   8I I8                     \n' +
        '                                                I8   8I I8                     \n' +
        "                                                I8, ,8' I8                     \n" +
        '                                                 "Y8P\'  I8                     ');
    console.log('\n' +
        `*** DX Wavefront Proxy Adaptor version ${dx_wfpa_version} started and listening on port ${listenPort} ***\n`);
}
exports.cliOptions = (0, cli_1.parseCLIOptions)();
bootstrap();
//# sourceMappingURL=main.js.map