"use strict";
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __metadata = (this && this.__metadata) || function (k, v) {
    if (typeof Reflect === "object" && typeof Reflect.metadata === "function") return Reflect.metadata(k, v);
};
var ConfigurationYamlService_1;
Object.defineProperty(exports, "__esModule", { value: true });
exports.ConfigurationYamlService = void 0;
const common_1 = require("@nestjs/common");
const yaml_1 = require("yaml");
const fs = require("node:fs");
const jsonschema_1 = require("jsonschema");
const path = require("node:path");
let ConfigurationYamlService = ConfigurationYamlService_1 = class ConfigurationYamlService {
    constructor() {
        this.logger = new common_1.Logger(ConfigurationYamlService_1.name);
        this.logger.log(`${ConfigurationYamlService_1.name} started`);
        const configurationSchemaLocation = path.join(__dirname, '..', '..', 'dx-wavefront-proxy-adapter.metric.config.json');
        this.logger.debug(`Loading configuration schema from ${configurationSchemaLocation}`);
        this.configurationSchema = JSON.parse(fs.readFileSync(configurationSchemaLocation).toString());
    }
    serialize(metricConfigs) {
        return (0, yaml_1.stringify)(metricConfigs, {
            defaultKeyType: 'PLAIN',
            defaultStringType: 'QUOTE_DOUBLE',
            aliasDuplicateObjects: false,
        });
    }
    validate(metricConfig) {
        const v = new jsonschema_1.Validator();
        const validatorResult = v.validate(metricConfig, this.configurationSchema);
        validatorResult.errors.forEach((error) => {
            this.logger.error('Error in configuration: ' + error.stack);
        });
        if (validatorResult.errors && validatorResult.errors.length > 0) {
            throw new Error(`Error validating metric config`);
        }
    }
    async parse(fileLocation) {
        return new Promise((resolve, reject) => {
            try {
                const parseResult = (0, yaml_1.parse)(fs.readFileSync(fileLocation).toString());
                if (parseResult && Array.isArray(parseResult)) {
                    try {
                        this.validate(parseResult);
                        resolve(parseResult);
                    }
                    catch (error) {
                        this.logger.error(`Error in configuration from ${fileLocation}`, error);
                        reject(error);
                    }
                }
                else {
                    this.logger.log(`Parse of configuration file (${fileLocation}) did not return an array, result on next line`, parseResult);
                    resolve([]);
                }
            }
            catch (error) {
                reject(error);
            }
        });
    }
};
exports.ConfigurationYamlService = ConfigurationYamlService;
exports.ConfigurationYamlService = ConfigurationYamlService = ConfigurationYamlService_1 = __decorate([
    (0, common_1.Injectable)(),
    __metadata("design:paramtypes", [])
], ConfigurationYamlService);
//# sourceMappingURL=configuration-yaml.service.js.map