"use strict";
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __metadata = (this && this.__metadata) || function (k, v) {
    if (typeof Reflect === "object" && typeof Reflect.metadata === "function") return Reflect.metadata(k, v);
};
var DXAccumulatorFactoryService_1;
Object.defineProperty(exports, "__esModule", { value: true });
exports.DXAccumulatorFactoryService = void 0;
const common_1 = require("@nestjs/common");
const dx_metric_type_1 = require("../../model/dx-metric-type");
const fluctuating_accumulator_1 = require("./fluctuating.accumulator");
const average_accumulator_1 = require("./average.accumulator");
const rate_accumulator_1 = require("./rate.accumulator");
const perinterval_accumulator_1 = require("./perinterval.accumulator");
const percentage_accumulator_1 = require("./percentage.accumulator");
const timestamp_accumulator_1 = require("./timestamp.accumulator");
const string_accumulator_1 = require("./string.accumulator");
const schedule_1 = require("@nestjs/schedule");
const dx_metric_reporter_service_1 = require("../dx-metric-reporter/dx-metric-reporter.service");
let DXAccumulatorFactoryService = DXAccumulatorFactoryService_1 = class DXAccumulatorFactoryService {
    constructor(dxMetricReporter) {
        this.dxMetricReporter = dxMetricReporter;
        this.accumulatorMap = new Map();
        this.logger = new common_1.Logger(DXAccumulatorFactoryService_1.name);
        this.logger.log('service started');
    }
    harvest() {
        const metricDatapoints = [];
        Array.from(this.accumulatorMap.values()).forEach((accumulator) => {
            metricDatapoints.push({
                metric: accumulator.dxMetric,
                datapoint: accumulator.collectAccumulatedDatapoint(),
            });
        });
        this.dxMetricReporter.reportMetrics(metricDatapoints);
    }
    hasAccumulator(key) {
        return this.accumulatorMap.has(key);
    }
    getAccumulator(key) {
        return this.accumulatorMap.get(key);
    }
    createAccumulatorFor(key, metricConfig, dxMetric) {
        const accumulator = this.instantiateAccumulatorFor(metricConfig, dxMetric);
        this.accumulatorMap.set(key, accumulator);
    }
    instantiateAccumulatorFor(metricConfig, dxMetric) {
        switch (dxMetric.type) {
            case dx_metric_type_1.DXMetricType.FLUCTUATING:
                return new fluctuating_accumulator_1.FluctuatingAccumulator(dxMetric, metricConfig.metric.metricScalar);
            case dx_metric_type_1.DXMetricType.AVERAGE:
                return new average_accumulator_1.AverageAccumulator(dxMetric, metricConfig.metric.metricScalar);
            case dx_metric_type_1.DXMetricType.RATE:
                return new rate_accumulator_1.RateAccumulator(dxMetric, metricConfig.metric.metricScalar);
            case dx_metric_type_1.DXMetricType.PER_INTERVAL:
                return new perinterval_accumulator_1.PerIntervalAccumulator(dxMetric, metricConfig.metric.metricScalar);
            case dx_metric_type_1.DXMetricType.PERCENTAGE:
                return new percentage_accumulator_1.PercentageAccumulator(dxMetric, metricConfig.metric.metricScalar);
            case dx_metric_type_1.DXMetricType.TIMESTAMP:
                return new timestamp_accumulator_1.TimestampAccumulator(dxMetric, metricConfig.metric.metricScalar);
            case dx_metric_type_1.DXMetricType.STRING:
                return new string_accumulator_1.StringAccumulator(dxMetric, 0);
            default:
                this.logger.log(`Unknown Metric Type (${dx_metric_type_1.DXMetricType[dxMetric.type]}), providing String accumulator.`);
                return new string_accumulator_1.StringAccumulator(dxMetric, 0);
        }
    }
};
exports.DXAccumulatorFactoryService = DXAccumulatorFactoryService;
__decorate([
    (0, schedule_1.Cron)(schedule_1.CronExpression.EVERY_MINUTE),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", []),
    __metadata("design:returntype", void 0)
], DXAccumulatorFactoryService.prototype, "harvest", null);
exports.DXAccumulatorFactoryService = DXAccumulatorFactoryService = DXAccumulatorFactoryService_1 = __decorate([
    (0, common_1.Injectable)(),
    __metadata("design:paramtypes", [dx_metric_reporter_service_1.DXMetricReporterService])
], DXAccumulatorFactoryService);
//# sourceMappingURL=dx-accumulator-factory.service.js.map