"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.RateAccumulator = void 0;
const dx_accumulator_factory_model_1 = require("./dx-accumulator-factory.model");
class RateAccumulator extends dx_accumulator_factory_model_1.BaseAccumulator {
    constructor(dxMetric, metricScalar) {
        super(dxMetric, metricScalar);
        this.values = [];
    }
    reportValue(valueAsString) {
        try {
            const value = this.getScaledValue(valueAsString);
            if (!this.min || value < this.min) {
                this.min = value;
            }
            if (!this.max || value > this.max) {
                this.max = value;
            }
            this.values.push(value);
        }
        catch (error) {
            this.logger.error(error);
        }
    }
    reset() {
        this.min = undefined;
        this.max = undefined;
        this.values = [];
    }
    collectAccumulatedDatapoint() {
        const datapoint = {
            min: this.values.length > 0 ? this.min : 0,
            max: this.values.length > 0 ? this.max : 0,
            value: Math.ceil(this.values.reduce((partialSum, a) => partialSum + a, 0) /
                this.values.length),
            count: this.values.length,
        };
        this.reset();
        return datapoint;
    }
}
exports.RateAccumulator = RateAccumulator;
//# sourceMappingURL=rate.accumulator.js.map