"use strict";
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __metadata = (this && this.__metadata) || function (k, v) {
    if (typeof Reflect === "object" && typeof Reflect.metadata === "function") return Reflect.metadata(k, v);
};
var DXMetricProcessorService_1;
Object.defineProperty(exports, "__esModule", { value: true });
exports.DXMetricProcessorService = void 0;
const common_1 = require("@nestjs/common");
const metric_safety_1 = require("../../util/metric-safety");
const dx_metric_type_1 = require("../../model/dx-metric-type");
const dx_accumulator_factory_service_1 = require("../dx-accumulator-factory/dx-accumulator-factory.service");
let DXMetricProcessorService = DXMetricProcessorService_1 = class DXMetricProcessorService {
    constructor(accumulatorFactory) {
        this.accumulatorFactory = accumulatorFactory;
        this.logger = new common_1.Logger(DXMetricProcessorService_1.name);
        this.logger.log('service started');
    }
    processMetricRecordForConfig(metricConfig, metricRecord) {
        if (metricConfig.enabled) {
            const hashableMetricRecord = {
                tags: metricRecord.tags,
                metric: metricRecord.metric,
            };
            const configRecordKey = JSON.stringify({
                c: metricConfig,
                hmr: hashableMetricRecord,
            });
            if (!this.accumulatorFactory.hasAccumulator(configRecordKey)) {
                const dxMetric = {
                    sourceName: this.getMetricSource(metricConfig.metric.source, metricRecord),
                    attributeName: this.getMetricAttribute(metricConfig.metric.resource, metricConfig.metric.metric, metricRecord),
                    type: dx_metric_type_1.DXMetricType[metricConfig.metric.metricType],
                    attributes: this.getAttributes(metricConfig, metricRecord),
                };
                this.accumulatorFactory.createAccumulatorFor(configRecordKey, metricConfig, dxMetric);
            }
            this.accumulatorFactory
                .getAccumulator(configRecordKey)
                .reportValue(metricRecord.rawValue);
        }
        else {
        }
    }
    getMetricSource(source, metricRecord) {
        return [
            'SuperDomain',
            this.interpolate(source.segment1, metricRecord),
            this.interpolate(source.segment2, metricRecord),
            this.interpolate(source.segment3, metricRecord),
        ].join('|');
    }
    getMetricAttribute(resource, metric, metricRecord) {
        const interpolatedResource = resource
            .map((resourcePart) => this.interpolate(resourcePart, metricRecord, metric_safety_1.escapeUnsafeResourceCharacters))
            .join('|');
        const interpolatedMetric = this.interpolate(metric, metricRecord);
        return resource.length > 0
            ? interpolatedResource + ':' + interpolatedMetric
            : interpolatedMetric;
    }
    interpolate(unterpolated, metricRecord, escapeFunction = metric_safety_1.escapeUnsafeMetricCharacters) {
        try {
            let interpolated = unterpolated;
            const interpolatables = unterpolated.match(DXMetricProcessorService_1.interpolationRegex);
            if (interpolatables) {
                interpolatables.forEach((interpolation) => {
                    interpolated = interpolated.replace(interpolation, this.getInterpolationValue(interpolation, metricRecord));
                });
            }
            return interpolated && interpolated.length > 0
                ? escapeFunction(interpolated)
                : '!empty!';
        }
        catch (error) {
            this.logger.error(error);
            return '!error!';
        }
    }
    getInterpolationValue(interpolation, metricRecord) {
        const realInterpolation = interpolation.substring(2, interpolation.length - 1);
        const interpolationSource = realInterpolation.substring(0, realInterpolation.indexOf(':'));
        const interpolationTarget = realInterpolation.substring(realInterpolation.indexOf(':') + 1, realInterpolation.length);
        switch (interpolationSource) {
            case 'tag':
                return this.getTagInterpolationValue(interpolationTarget, metricRecord.tags);
            case 'mp':
                return this.getMetricPartInterpolationValue(interpolationTarget, metricRecord.metric);
            case 'mmps':
                return this.getMultipleMetricPartsInterpolationValue(interpolationTarget, metricRecord.metric);
            default:
                return 'unknown';
        }
    }
    getTagInterpolationValue(interpolationTarget, tags) {
        return tags[interpolationTarget] && tags[interpolationTarget].length > 0
            ? tags[interpolationTarget]
            : 'unknown';
    }
    getMetricPartInterpolationValue(interpolationTarget, metric) {
        const metricPartIndex = parseInt(interpolationTarget);
        if (metricPartIndex == 0)
            return metric;
        const metricParts = metric.split('.');
        if (metricPartIndex < 0) {
            return metricParts[metricParts.length + metricPartIndex];
        }
        if (metricPartIndex <= metricParts.length) {
            return metricParts[metricPartIndex - 1];
        }
    }
    getMultipleMetricPartsInterpolationValue(interpolationTarget, metric) {
        const metricParts = metric.split('.');
        const multipartInterpolationParameters = interpolationTarget.split(':');
        if (multipartInterpolationParameters.length != 3) {
            this.logger.error(`multiple-metric-parts interpolation did not have 3 parameters: ${interpolationTarget}`);
            return '!error!';
        }
        const startIndex = parseInt(multipartInterpolationParameters[0]);
        if (Number.isNaN(startIndex) || startIndex == 0) {
            this.logger.error(`multiple-metric-parts interpolation start index must be a number > 0: ${startIndex}`);
            return '!error!';
        }
        if (startIndex > metricParts.length) {
            this.logger.error(`multiple-metric-parts interpolation start index ${startIndex} must be less than # of metric parts (${metricParts.length})`);
            return '!error!';
        }
        let endIndex = -1;
        if (multipartInterpolationParameters[1] == '*') {
            endIndex = metricParts.length + 1;
        }
        if (multipartInterpolationParameters[1].startsWith('+')) {
            endIndex = startIndex + parseInt(multipartInterpolationParameters[1]) + 1;
        }
        if (multipartInterpolationParameters[1].startsWith('-')) {
            endIndex =
                metricParts.length + parseInt(multipartInterpolationParameters[1]) + 1;
            if (endIndex <= startIndex)
                endIndex = startIndex + 1;
        }
        if (endIndex < startIndex) {
            this.logger.error(`multiple-metric-parts interpolation endIndex (${endIndex}) is less than startIndex (${startIndex})`);
            return '!error!';
        }
        const slicedParts = metricParts.slice(startIndex - 1, endIndex - 1);
        return slicedParts.join(multipartInterpolationParameters[2]);
    }
    getAttributes(metricConfig, metricRecord) {
        const attributes = {};
        if (metricConfig.tagsAsAttributesStrategy &&
            metricConfig.tagsAsAttributesStrategy == 'ALL') {
            Object.keys(metricRecord.tags).forEach((tag) => {
                attributes[tag] = metricRecord.tags[tag];
            });
        }
        else {
            (metricConfig.tagsAsAttributes ?? []).forEach((tag) => {
                attributes[tag] = metricRecord.tags[tag];
            });
        }
        Object.keys(metricConfig.additionalAttributes).forEach((tag) => {
            attributes[tag] = this.interpolate(metricConfig.additionalAttributes[tag], metricRecord);
        });
        attributes['wavefront_metric'] = metricRecord.metric;
        return attributes;
    }
};
exports.DXMetricProcessorService = DXMetricProcessorService;
DXMetricProcessorService.interpolationRegex = /\${(.+?)}/g;
exports.DXMetricProcessorService = DXMetricProcessorService = DXMetricProcessorService_1 = __decorate([
    (0, common_1.Injectable)(),
    __metadata("design:paramtypes", [dx_accumulator_factory_service_1.DXAccumulatorFactoryService])
], DXMetricProcessorService);
//# sourceMappingURL=dx-metric-processor.service.js.map