"use strict";
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __metadata = (this && this.__metadata) || function (k, v) {
    if (typeof Reflect === "object" && typeof Reflect.metadata === "function") return Reflect.metadata(k, v);
};
var DXMetricReporterService_1;
Object.defineProperty(exports, "__esModule", { value: true });
exports.DXMetricReporterService = void 0;
const common_1 = require("@nestjs/common");
const nass_service_1 = require("../nass/nass.service");
const array_split_1 = require("@blakek/array-split");
const promise_pool_1 = require("@supercharge/promise-pool");
let DXMetricReporterService = DXMetricReporterService_1 = class DXMetricReporterService {
    constructor(nass) {
        this.nass = nass;
        this.metricIdRegistrationMap = new Map();
        this.logger = new common_1.Logger(DXMetricReporterService_1.name);
        this.logger.log('service started');
    }
    async reportMetrics(metricDatapoints) {
        const metricsToRegister = [];
        metricDatapoints.forEach((metricDatapoint) => {
            if (!this.metricIdRegistrationMap.has(this.getMetricRegistrationKey(metricDatapoint.metric))) {
                metricsToRegister.push(metricDatapoint.metric);
            }
        });
        if (metricsToRegister.length > 0) {
            this.logger.log(`Attempting to register ${metricsToRegister.length} metrics`);
            const metricsToRegisterChunks = (0, array_split_1.chunk)(3072, metricsToRegister);
            await promise_pool_1.PromisePool.for(metricsToRegisterChunks)
                .handleError((error) => {
                this.logger.error('Error registering metric chunk', error);
            })
                .withConcurrency(5)
                .process(async (metricsToRegisterChunk) => {
                return this.nass
                    .registerMetrics(metricsToRegisterChunk)
                    .then((registeredMetricsChunk) => {
                    registeredMetricsChunk.metrics.forEach((registeredMetric) => {
                        this.metricIdRegistrationMap.set(this.getRegisteredMetricKey(registeredMetric), registeredMetric.id);
                    });
                    this.logger.debug('registered metric chunk.');
                });
            });
        }
        const metricValues = [];
        const timestamp = Math.ceil(new Date().getTime() / 1000);
        metricDatapoints.forEach((metricDatapoint) => {
            if (this.metricIdRegistrationMap.has(this.getMetricRegistrationKey(metricDatapoint.metric))) {
                metricValues.push({
                    metricId: this.metricIdRegistrationMap.get(this.getMetricRegistrationKey(metricDatapoint.metric)),
                    timestampSeconds: timestamp,
                    interval: 60,
                    value: metricDatapoint.datapoint.value,
                    count: metricDatapoint.datapoint.count,
                    min: metricDatapoint.datapoint.min,
                    max: metricDatapoint.datapoint.max,
                });
            }
            else {
                this.logger.warn(`No metric id for ${JSON.stringify(metricDatapoint.metric)}`);
            }
        });
        if (metricValues.length > 0) {
            const metricValuesChunks = (0, array_split_1.chunk)(256, metricValues);
            this.logger.debug(`Attempting to store ${metricValues.length} metric values`);
            promise_pool_1.PromisePool.for(metricValuesChunks)
                .handleError((error) => {
                this.logger.error('Error storing metric value chunk', error);
            })
                .withConcurrency(5)
                .process(async (metricValuesChunk) => {
                return this.nass.storeMetrics(metricValuesChunk).then(() => {
                    this.logger.debug('stored metric chunk.');
                });
            })
                .then(() => {
                this.logger.log('Metrics stored.');
            });
        }
    }
    getMetricRegistrationKey(metric) {
        return metric.sourceName + ':' + metric.attributeName;
    }
    getRegisteredMetricKey(registeredMetric) {
        return registeredMetric.sourceName + ':' + registeredMetric.attributeName;
    }
};
exports.DXMetricReporterService = DXMetricReporterService;
exports.DXMetricReporterService = DXMetricReporterService = DXMetricReporterService_1 = __decorate([
    (0, common_1.Injectable)(),
    __metadata("design:paramtypes", [nass_service_1.NassService])
], DXMetricReporterService);
//# sourceMappingURL=dx-metric-reporter.service.js.map