"use strict";
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __metadata = (this && this.__metadata) || function (k, v) {
    if (typeof Reflect === "object" && typeof Reflect.metadata === "function") return Reflect.metadata(k, v);
};
var MetricRecordConfigurationService_1;
Object.defineProperty(exports, "__esModule", { value: true });
exports.MetricRecordConfigurationService = void 0;
const common_1 = require("@nestjs/common");
const schedule_1 = require("@nestjs/schedule");
const config_1 = require("@nestjs/config");
const sort_1 = require("../../util/sort");
const fs = require("node:fs");
const path = require("node:path");
const configuration_yaml_service_1 = require("../configuration-yaml/configuration-yaml.service");
const lodash_1 = require("lodash");
const event_emitter_1 = require("@nestjs/event-emitter");
let MetricRecordConfigurationService = MetricRecordConfigurationService_1 = class MetricRecordConfigurationService {
    constructor(configuration, configYAML, eventEmitter) {
        this.configuration = configuration;
        this.configYAML = configYAML;
        this.eventEmitter = eventEmitter;
        this.logger = new common_1.Logger(MetricRecordConfigurationService_1.name);
        this.proposedConfigurations = new Map();
        this.proposedConfigurationNames = new Set();
        this.spoolDisabled = false;
        this.discoveryIndex = 0;
        this.configurationsMap = new Map();
        this.metricRecordConfigurationCacheHits = 0;
        this.metricRecordConfigurationCacheMisses = 0;
        this.recordConfigurationCache = new Map();
        this.logger.log('Loading Configurations.');
        this.loadConfigurations()
            .then(() => {
            this.logger.log('Initial Configuration load complete.');
            this.initializeConfigurationChangeWatcher();
        })
            .catch((error) => {
            this.logger.error('Error loading initial configuration.', error);
        });
        this.logger.log('service started');
    }
    initializeConfigurationChangeWatcher() {
        fs.watch(this.configuration.get('adapter.configurations.directory')).on('change', (eventType, filename) => {
            if (filename.toString().endsWith('.metric.config.yaml')) {
                this.logger.log(`Detected configuration change: ${eventType} on ${filename}`);
                this.loadConfigurationFile(this.configuration.get('adapter.configurations.directory'), filename.toString());
            }
        });
    }
    async loadConfigurations() {
        const configurationDirectoryLocation = this.configuration.get('adapter.configurations.directory');
        const configurationDirectoryAbsolutePath = path.resolve(configurationDirectoryLocation);
        this.logger.debug(`Found configuration for configuration directory: ${configurationDirectoryLocation}`);
        this.logger.debug(`Configuration directory absolute path: ${configurationDirectoryAbsolutePath}`);
        await fs.promises
            .access(configurationDirectoryAbsolutePath, fs.constants.W_OK)
            .then(() => {
            fs.promises
                .readdir(configurationDirectoryAbsolutePath)
                .then((results) => {
                const configFiles = results.filter((result) => result.endsWith('.metric.config.yaml'));
                if (configFiles.length > 0) {
                    configFiles.forEach(async (configFile) => {
                        await this.loadConfigurationFile(configurationDirectoryAbsolutePath, configFile);
                    });
                }
                else {
                    this.logger.warn(`No configuration files found in ${configurationDirectoryAbsolutePath}`);
                    this.logger.warn(`configuration files must end with ".metric.config.yaml" suffix`);
                }
            })
                .catch((error) => {
                this.logger.error(error);
                throw new error('Error loading configuration files', error);
            });
        })
            .catch(() => {
            this.logger.error(`Configuration Directory does not exist or is not accessible: ${configurationDirectoryAbsolutePath}`);
        });
    }
    async loadConfigurationFile(configurationDirectoryAbsolutePath, configFile) {
        const absConfigFileLocation = path.join(configurationDirectoryAbsolutePath, configFile);
        this.logger.log(`Loading configuration file: ${absConfigFileLocation}`);
        try {
            const config = await this.configYAML.parse(absConfigFileLocation);
            this.logger.log(`Loaded ${config.length} metric configurations from ${absConfigFileLocation}`);
            this.configurationsMap.set(configFile, config);
            this.eventEmitter.emit('configuration.changed');
        }
        catch (error) {
            this.logger.error(`Error loading configuration file, no configurations from file ${absConfigFileLocation} were loaded.`);
        }
    }
    spoolSampleConfigurations() {
        if (this.proposedConfigurations.size > 0) {
            this.logger.debug(`there are ${this.proposedConfigurations.size} sample configurations`);
            const configurationsToSpool = this.proposedConfigurations;
            this.proposedConfigurations = new Map();
            const discoveryDirectoryLocation = this.configuration.get('adapter.discovery.directory');
            const discoveryDirectoryAbsolutePath = path.resolve(discoveryDirectoryLocation);
            this.logger.debug(`found configuration for discovery directory: ${discoveryDirectoryLocation}`);
            this.logger.debug(`discovery directory absolute path: ${discoveryDirectoryAbsolutePath}`);
            fs.promises
                .access(discoveryDirectoryAbsolutePath, fs.constants.W_OK)
                .then(() => {
                const discoveryFilename = `metrics.discovered.${this.discoveryIndex++}.yaml`;
                fs.promises
                    .writeFile(path.join(discoveryDirectoryAbsolutePath, discoveryFilename), this.configYAML.serialize([...configurationsToSpool.values()]))
                    .then(() => {
                    this.logger.log(`Spooled discovered metrics to ${discoveryFilename}`);
                })
                    .catch((error) => {
                    this.logger.log(`Error spooling discovered metrics to ${discoveryFilename}`);
                    this.logger.log(error);
                });
            })
                .catch(() => {
                this.logger.warn(`Could not access discovery directory: ${discoveryDirectoryAbsolutePath}`);
                if (fs.existsSync(discoveryDirectoryAbsolutePath)) {
                    this.logger.error('Discovery directory exists, probably a permissions problem.');
                    this.spoolDisabled = true;
                    this.logger.error('Discovered Configuration spool disabled.');
                }
                else {
                    try {
                        this.logger.warn(`Configured discovery directory does not exist: ${discoveryDirectoryAbsolutePath}`);
                        fs.mkdirSync(discoveryDirectoryAbsolutePath);
                        this.logger.log('Created discovery directory');
                    }
                    catch (e) {
                        this.logger.error('Error creating discovery directory');
                        this.logger.error('Discovered Configuration spool disabled.');
                    }
                }
            });
        }
    }
    getConfigurations(metricRecord) {
        const matchingConfigurations = this.getMatchingConfigurations(metricRecord);
        if (matchingConfigurations.length == 0) {
            this.proposeSampleConfiguration(metricRecord);
        }
        return matchingConfigurations;
    }
    handleOrderCreatedEvent() {
        this.logger.log('Configuration changed: clearing record configuration cache');
        this.recordConfigurationCache.clear();
    }
    reportRecordConfigurationCacheStatistics() {
        this.logger.log(`Record Configuration Hits/Misses: ${this.metricRecordConfigurationCacheHits}/${this.metricRecordConfigurationCacheMisses}`);
        this.metricRecordConfigurationCacheMisses = 0;
        this.metricRecordConfigurationCacheHits = 0;
    }
    getMatchingConfigurations(metricRecord) {
        const metricRecordCacheKey = this.getMetricRecordCacheKey(metricRecord);
        if (this.recordConfigurationCache.has(metricRecordCacheKey)) {
            this.metricRecordConfigurationCacheHits++;
            return this.recordConfigurationCache.get(metricRecordCacheKey);
        }
        else {
            this.metricRecordConfigurationCacheMisses++;
            const matchingConfigurations = [];
            this.configurationsMap.forEach((configurations) => {
                configurations.forEach((metricConfig) => {
                    if (this.configurationMatches(metricConfig, metricRecord)) {
                        matchingConfigurations.push(metricConfig);
                    }
                });
            });
            this.recordConfigurationCache.set(metricRecordCacheKey, matchingConfigurations);
            return matchingConfigurations;
        }
    }
    getMetricRecordCacheKey(metricRecord) {
        const sortedTags = (0, sort_1.sort)(Object.keys(metricRecord.tags));
        return `${metricRecord.metric}[${sortedTags.join(',')}]`;
    }
    proposeSampleConfiguration(metricRecord) {
        const sortedTags = (0, sort_1.sort)(Object.keys(metricRecord.tags));
        const metricConfigName = `${metricRecord.metric}[${sortedTags.join(',')}]`;
        if (!this.alreadyProposed(metricConfigName)) {
            const proposedMetricConfig = {
                description: `Sample Config For ${metricConfigName}`,
                enabled: true,
                match: {
                    metric: {
                        matchType: 'EXACT',
                        match: metricRecord.metric,
                    },
                    tag: {
                        matchType: 'ATLEAST',
                        tags: sortedTags,
                    },
                },
                metric: {
                    source: {
                        segment1: '${tag:source}',
                        segment2: 'WaveFront Proxy Adaptor',
                        segment3: '${tag:source}',
                    },
                    resource: this.generateResource(metricRecord),
                    metric: this.generateMetric(metricRecord.metric),
                    metricType: 'FLUCTUATING',
                    metricScalar: 1,
                },
                tagsAsAttributesStrategy: 'ALL',
                tagsAsAttributes: sortedTags,
                additionalAttributes: {
                    'created-by-wfpa': 'true',
                },
                sampleValue: metricRecord.rawValue,
                sampleTags: metricRecord.tags,
            };
            this.addToProposals(metricConfigName, proposedMetricConfig);
        }
    }
    addToProposals(metricConfigName, proposedMetricConfig) {
        this.proposedConfigurationNames.add(metricConfigName);
        this.proposedConfigurations.set(metricConfigName, proposedMetricConfig);
    }
    alreadyProposed(metricConfigName) {
        return this.proposedConfigurationNames.has(metricConfigName);
    }
    generateResource(metricRecord) {
        const resourceParts = Object.keys(metricRecord.tags)
            .filter((tagName) => tagName !== 'source')
            .map((tagName) => '${tag:' + tagName + '}');
        const metricParts = metricRecord.metric.split('.');
        if (metricParts.length > 1) {
            resourceParts.push(...metricParts.slice(0, metricParts.length - 1));
            return resourceParts;
        }
        else {
            return resourceParts;
        }
    }
    generateMetric(metric) {
        const metricParts = metric.split('.');
        if (metricParts.length > 1) {
            return metricParts[metricParts.length - 1];
        }
        else {
            return metric;
        }
    }
    configurationMatches(metricConfig, metricRecord) {
        return (this.metricMatches(metricConfig, metricRecord) &&
            this.tagMatches(metricConfig, metricRecord));
    }
    metricMatches(metricConfig, metricRecord) {
        const mm = metricConfig.match.metric.matchType === 'EXACT'
            ? metricConfig.match.metric.match === metricRecord.metric
            : metricRecord.metric.match(metricConfig.match.metric.match);
        return mm;
    }
    tagMatches(metricConfig, metricRecord) {
        return metricConfig.match.tag.matchType === 'EXACT'
            ? this.exactTagMatch(metricConfig.match.tag.tags, metricRecord.tags)
            : this.atleastTagMatch(metricConfig.match.tag.tags, metricRecord.tags);
    }
    exactTagMatch(configTags, recordTags) {
        return lodash_1.default.isEmpty(lodash_1.default.xor(configTags, Object.keys(recordTags)));
    }
    atleastTagMatch(configTags, recordTags) {
        for (const requiredTag of configTags) {
            if (!Object.keys(recordTags).includes(requiredTag))
                return false;
        }
        return true;
    }
};
exports.MetricRecordConfigurationService = MetricRecordConfigurationService;
__decorate([
    (0, schedule_1.Cron)(schedule_1.CronExpression.EVERY_MINUTE),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", []),
    __metadata("design:returntype", void 0)
], MetricRecordConfigurationService.prototype, "spoolSampleConfigurations", null);
__decorate([
    (0, event_emitter_1.OnEvent)('configuration.changed'),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", []),
    __metadata("design:returntype", void 0)
], MetricRecordConfigurationService.prototype, "handleOrderCreatedEvent", null);
__decorate([
    (0, schedule_1.Cron)(schedule_1.CronExpression.EVERY_MINUTE),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", []),
    __metadata("design:returntype", void 0)
], MetricRecordConfigurationService.prototype, "reportRecordConfigurationCacheStatistics", null);
exports.MetricRecordConfigurationService = MetricRecordConfigurationService = MetricRecordConfigurationService_1 = __decorate([
    (0, common_1.Injectable)(),
    __metadata("design:paramtypes", [config_1.ConfigService,
        configuration_yaml_service_1.ConfigurationYamlService,
        event_emitter_1.EventEmitter2])
], MetricRecordConfigurationService);
//# sourceMappingURL=metric-record-configuration.service.js.map