"use strict";
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __metadata = (this && this.__metadata) || function (k, v) {
    if (typeof Reflect === "object" && typeof Reflect.metadata === "function") return Reflect.metadata(k, v);
};
var NassService_1;
Object.defineProperty(exports, "__esModule", { value: true });
exports.NassService = void 0;
const common_1 = require("@nestjs/common");
const config_1 = require("@nestjs/config");
const dx_do_1 = require("dx-do");
const fs = require("node:fs");
const SimpleLog_1 = require("dx-do/dist/util/log/SimpleLog");
const Signaleton_1 = require("dx-do/dist/util/log/Signaleton");
const simple_nest_log_1 = require("../../util/log/simple-nest-log");
let NassService = NassService_1 = class NassService {
    constructor(configService) {
        this.configService = configService;
        this.metricTypeMap = new Map();
        this.metricIdMap = new Map();
        this.logger = new common_1.Logger(NassService_1.name);
        try {
            const dxTenantConfigurationFile = configService.get('dx.tenantConfigurationFile');
            if (!fs.existsSync(dxTenantConfigurationFile)) {
                this.logger.error(`DX Tenant configuration file not found: ${dxTenantConfigurationFile}`);
                this.logger.error(`You can set this using the DX_TENANT_CONFIG_FILE environment variable.`);
                throw new Error(`DX Tenant configuration file not found: ${dxTenantConfigurationFile}`);
            }
            const dxSaasConfiguration = JSON.parse(fs.readFileSync(dxTenantConfigurationFile).toString());
            const dxDoConfiguration = {
                dxConfiguration: dxSaasConfiguration,
                mongoConfiguration: null,
            };
            this.dxSaasService = new dx_do_1.DXSaaSService(dxDoConfiguration, new simple_nest_log_1.SimpleNestLog(this.logger));
            this.dxSaasService
                .ping()
                .then(() => {
                this.logger.log('Ping to DX SaaS Tenant succeeded.');
            })
                .catch((error) => this.logger.error('Ping to DX SaaS Tenant failed, your configuration may not be correct or the service may be down for maintenance.', error));
            this.dxNassService = new dx_do_1.NASSService(this.dxSaasService, new SimpleLog_1.SignaleSimpleLog(Signaleton_1.Signaleton.getInstance()));
        }
        catch (error) {
            this.logger.error('Error setting up DX Tenant NassService', error);
            throw error;
        }
        this.logger.log('service started');
    }
    getConfig() {
        return this.configService.get('dx.tenantConfigurationFile');
    }
    async storeMetric(sourceName, attributeName, type, min, max, value, count, interval, tags) {
        const metricId = await this.getMetricId(sourceName, attributeName, type, tags);
        const timestamp = Math.ceil(new Date().getTime() / 1000);
        return this.dxNassService.storeMetricValue([
            {
                metricId: metricId,
                max: max,
                min: min,
                count: count,
                interval: interval,
                value: value,
                timestampSeconds: timestamp,
            },
        ]);
    }
    async storeMetrics(metricValues) {
        return this.dxNassService.storeMetricValue(metricValues);
    }
    async getMetricId(sourceName, attributeName, type, tags) {
        const metricKey = `${sourceName}-${attributeName}`;
        if (this.metricTypeMap.has(metricKey)) {
            if (this.metricTypeMap.get(metricKey) === type) {
                return this.metricIdMap.get(metricKey);
            }
            else {
                throw new Error('bad metric type');
            }
        }
        else {
            const metricRegistration = await this.registerMetric(sourceName, attributeName, type, tags);
            this.metricTypeMap.set(metricKey, metricRegistration.metrics[0].type);
            this.metricIdMap.set(metricKey, metricRegistration.metrics[0].id);
            return metricRegistration.metrics[0].id;
        }
    }
    async registerMetrics(metricRegistrations) {
        return this.dxNassService.registerMetrics(metricRegistrations);
    }
    async registerMetric(sourceName, attributeName, type, tags) {
        return this.dxNassService.registerMetrics([
            {
                sourceName,
                attributeName,
                type,
                attributes: tags,
            },
        ]);
    }
};
exports.NassService = NassService;
exports.NassService = NassService = NassService_1 = __decorate([
    (0, common_1.Injectable)(),
    __metadata("design:paramtypes", [config_1.ConfigService])
], NassService);
//# sourceMappingURL=nass.service.js.map