"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.cliOptionsHelpConfig = exports.cliOptionsConfig = void 0;
exports.parseCLIOptions = parseCLIOptions;
const ts_command_line_args_1 = require("ts-command-line-args");
const dxwfpaschema = require("../../dx-wavefront-proxy-adapter.metric.config.json");
exports.cliOptionsConfig = {
    dumpConfigurationSchema: {
        type: Boolean,
        optional: true,
        description: 'Dumps a json schema which can be used to help edit configuration files.',
    },
    listenPort: {
        type: Number,
        optional: true,
        description: 'e.g., 2401; server listen port',
    },
    requestBodyMaxSize: {
        type: String,
        optional: true,
        description: "e.g., '17mb'; Maximum payload size from Wavefront proxy",
    },
    logLevels: {
        type: String,
        optional: true,
        description: "comma separated list of log levels; e.g.,'error,warn,log,debug'",
    },
    dxTenantConfig: {
        type: String,
        optional: true,
        description: 'location of dx-do v3 configuration file.',
    },
    discoveryDirectory: {
        type: String,
        optional: true,
        alias: 'd',
        description: 'directory in which discovered metrics sample configurations will be stored.',
    },
    configurationsDirectory: {
        type: String,
        optional: true,
        alias: 'c',
        description: 'directory in which active configurations are stored.',
    },
    help: {
        type: Boolean,
        optional: true,
        alias: 'h',
        description: 'Prints this usage guide',
    },
};
exports.cliOptionsHelpConfig = {
    helpArg: 'help',
    headerContentSections: [
        {
            header: 'DX Wavefront Proxy Adapater',
            content: 'Experimental Unsupported Metric Transport and Translation from Wavefront Proxy to the Broadcom DX AIOps Platform',
        },
    ],
    footerContentSections: [
        {
            header: 'Summary',
            content: `See dx-do v3 configuration format here: https://www.npmjs.com/package/dx-do`,
        },
    ],
};
function parseCLIOptions() {
    const options = (0, ts_command_line_args_1.parse)({
        dumpConfigurationSchema: {
            type: Boolean,
            optional: true,
            description: 'Dumps a json schema which can be used to help edit configuration files.',
        },
        listenPort: {
            type: Number,
            optional: true,
            description: 'e.g., 2401; server listen port',
        },
        requestBodyMaxSize: {
            type: String,
            optional: true,
            description: "e.g., '17mb'; Maximum payload size from Wavefront proxy",
        },
        logLevels: {
            type: String,
            optional: true,
            description: "comma separated list of log levels; e.g.,'error,warn,log,debug'",
        },
        dxTenantConfig: {
            type: String,
            optional: true,
            description: 'location of dx-do v3 configuration file.',
        },
        discoveryDirectory: {
            type: String,
            optional: true,
            alias: 'd',
            description: 'directory in which discovered metrics sample configurations will be stored.',
        },
        configurationsDirectory: {
            type: String,
            optional: true,
            alias: 'c',
            description: 'directory in which active configurations are stored.',
        },
        help: {
            type: Boolean,
            optional: true,
            alias: 'h',
            description: 'Prints this usage guide',
        },
    }, {
        helpArg: 'help',
        headerContentSections: [
            {
                header: 'DX Wavefront Proxy Adapter',
                content: 'Experimental Unsupported Metric Transport and Translation from Wavefront Proxy to the Broadcom DX AIOps Platform',
            },
        ],
        footerContentSections: [
            {
                header: 'Summary',
                content: `See dx-do v3 configuration format here: https://www.npmjs.com/package/dx-do`,
            },
        ],
    });
    if (options.dumpConfigurationSchema) {
        console.log(JSON.stringify(dxwfpaschema, null, 2));
        process.exit(0);
    }
    return options;
}
//# sourceMappingURL=cli.js.map